﻿-- debug mode off by default
local debugMode = false;
local isReallyOnline = {};


function GuildWho_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0);
end


-- debug message function, only visible in debug mode
function GuildWho_Debug(msg)
	if debugMode then
		DEFAULT_CHAT_FRAME:AddMessage("## "..msg, 1, 1, 0);
	end
end


-- function to toggle debug mode
function GuildWho_DebugMode()
	if not debugMode then
		debugMode = true;
		GuildWho_Print("GuildWho: Debug mode ON.");
	else
		debugMode = false;
		GuildWho_Print("GuildWho: Debug mode OFF.");
	end
end


function GuildWho_GetLookup(searchStr)
	local lookup, mainChar;
	local showOffline = GetGuildRosterShowOffline();
	
	SetGuildRosterShowOffline(true);
	GuildWho_Debug("Showing offline members.");
	
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
		if string.lower(searchStr) == string.lower(name) then
			lookup = name;
			GuildWho_Debug("lookup = "..lookup);
			
			SetGuildRosterShowOffline(showOffline);
			GuildWho_Debug("Restoring display of offline members.");
			
			-- search string matches an online character, so no need to look any further
			if isReallyOnline[name] then
				return lookup, name;
			end
			
			-- if this character's officer note is EPGPs or blank it's the main, else it's the name of the main
			if string.find(note, ",") or note == "" then
				mainChar = name;
			else
				mainChar = note;
			end
			GuildWho_Debug("mainChar = "..mainChar);
			
			break;
		end
	end
	return lookup, GuildWho_GetAlt(mainChar);
end


function GuildWho_GetAlt(mainChar)
	local altChar;
	local showOffline = GetGuildRosterShowOffline();
	
	SetGuildRosterShowOffline(true);
	GuildWho_Debug("Showing offline members.");
	
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
		if (note == mainChar or name == mainChar) and isReallyOnline[name] then
			GuildWho_Debug("alt = "..name);
			altChar = name;
			break;
		end
	end
	
	SetGuildRosterShowOffline(showOffline);
	GuildWho_Debug("Restoring display of offline members.");
	
	return altChar;
end


function GuildWho_UpdateGuildRoster()
	local showOffline = GetGuildRosterShowOffline();
	
	SetGuildRosterShowOffline(true);
	GuildWho_Debug("Showing offline members.");
	
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i);
		isReallyOnline[name] = online or false;
	end
	
	SetGuildRosterShowOffline(showOffline);
	GuildWho_Debug("Restoring display of offline members.");
end


-- match the CHAT_MSG_SYSTEM against online/offline patterns
function GuildWho_ParseSystemMessage(msg)
	local name;
	local online = false;
	local memberChange = false;

	local patterns = {
		"^"..ERR_FRIEND_ONLINE_SS:format("([^|]+)", ""):gsub("%.", "%%."):gsub("%[%]", ".+").."$",
		"^"..ERR_FRIEND_OFFLINE_S:format("(%S+)"):gsub("%.", "%%.").."$",
		"^"..ERR_GUILD_JOIN_S:format("(%S+)"):gsub("%.", "%%.").."$",
		"^"..ERR_GUILD_LEAVE_S:format("(%S+)"):gsub("%.", "%%.").."$",
		"^"..ERR_GUILD_REMOVE_SS:format("(%S+)", "(%S+)"):gsub("%.", "%%.").."$",
	}
	
	for k, v in pairs(patterns) do
		name = string.match(msg, v);
		if name then
			if k == 1 or k == 3 then
				if k == 3 then
					memberChange = true;
				end
				online = 1;
			elseif k == 4 or k == 5 then
				isReallyOnline[name] = nil;
				memberChange = true;
			end
			break;
		end
	end
	
	return name, online, memberChange;
end


function GuildWhoOnline_SlashHandler(msg)
	if not msg or msg == "" then
		GuildWho_Print("Usage:");
		GuildWho_Print("'/gwho | /online character' - find who character is online as\n");
		GuildWho_Print("'/alt character' - list all of character's alts\n");
		GuildWho_Print("'/gwho debug' - toggle debug messages.");
	elseif string.lower(msg) == "debug" then
		GuildWho_DebugMode();
	else
		local lookup, alt = GuildWho_GetLookup(msg);
		
		-- no character match found
		if not lookup then
			GuildWho_Print("No character named "..string.upper(string.sub(msg, 1, 1))..string.lower(string.sub(msg, 2)).." found in guild.")
			return;
		end
		
		-- print result
		if alt then
			if lookup ~= alt then
				asAlt = " as ".."|Hplayer:"..alt.."|h["..alt.."]|h";
			else
				lookup = "|Hplayer:"..lookup.."|h["..lookup.."]|h";
				asAlt = "";
			end
			GuildWho_Print(lookup.." is online"..asAlt..".");
		else
			GuildWho_Print(lookup.." is offline.");
		end
	end
end


function GuildWhoAlt_SlashHandler(msg)
	if not msg or msg == "" then
		GuildWho_Print("Usage:");
		GuildWho_Print("'/gwho | /online character' - find who character is online as\n");
		GuildWho_Print("'/alt character' - list all of character's alts\n");
		GuildWho_Print("'/gwho debug' - toggle debug messages.");
	else
		local lookup, mainChar;
		-- local hasAlts = false;
		local alts = {};
		
		local showOffline = GetGuildRosterShowOffline();
		
		-- show offline members to be able to parse them
		SetGuildRosterShowOffline(true);
		GuildWho_Debug("Showing offline members.");
		
		for i = 1, GetNumGuildMembers(true) do
			local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
			if string.lower(msg) == string.lower(name) then
				lookup = name;
				
				if string.find(note, ",") or note == "" then
					mainChar = name;
				else
					mainChar = note;
				end
				GuildWho_Debug("mainChar = "..mainChar);
			end
		end
		
		for i = 1, GetNumGuildMembers(true) do
			local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
			if (note == mainChar or name == mainChar) and name ~= lookup then
				hasAlts = true;
				-- if isReallyOnline[name] then
					-- name = "|Hplayer:"..name.."|h["..name.."]|h (online)";
				-- end
				alts[#alts + 1] = name;
				GuildWho_Debug("Found alt "..name);
			end
		end
		
		-- restore old "show offline members" state
		SetGuildRosterShowOffline(showOffline);
		GuildWho_Debug("Restoring display of offline members.");
		
		if not lookup then
			GuildWho_Print("No character named "..string.upper(string.sub(msg, 1, 1))..string.lower(string.sub(msg, 2)).." found in guild.")
		elseif #alts == 0 then
			GuildWho_Print(lookup.." has no known alts.");
		else
			GuildWho_Print("Known alts of "..lookup..":");
			for k, v in pairs(alts) do
				if isReallyOnline[v] then
					v = "|Hplayer:"..v.."|h["..v.."]|h (online)";
				end
				GuildWho_Print("  "..v);
			end
		end
	end
end


SlashCmdList["GWHO_ONLINE"] = GuildWhoOnline_SlashHandler
SLASH_GWHO_ONLINE1 = "/gwho";
SLASH_GWHO_ONLINE2 = "/online";

SlashCmdList["GWHO_ALT"] = GuildWhoAlt_SlashHandler
SLASH_GWHO_ALT1 = "/alts";


-- add a "List alts" entry to unit popup menus
hooksecurefunc("UnitPopup_OnClick", function()
	local dropdownFrame = getglobal(UIDROPDOWNMENU_INIT_MENU);
	local button = this.value;
	local name = dropdownFrame.name;
	
	if button == "GW_LISTALTS" then
		if name then
			GuildWhoAlt_SlashHandler(name);
		end
	end
end);


-- display only for guild members
hooksecurefunc("UnitPopup_HideButtons", function()
	local dropdownMenu = getglobal(UIDROPDOWNMENU_INIT_MENU);
	local showEntry;
	
	for index, value in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "GW_LISTALTS" then
			showEntry = 0;
			for character, isOnline in pairs(isReallyOnline) do
				if character == dropdownMenu.name then
					showEntry = 1;
				end
			end
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = showEntry;
			break;
		end
	end
end);

UnitPopupButtons["GW_LISTALTS"] = { text = "List alts", dist = 0, func = UnitPopup_OnClick};
tinsert(UnitPopupMenus["FRIEND"], #UnitPopupMenus["FRIEND"] - 1, "GW_LISTALTS");


local frame = CreateFrame("Frame");
frame:SetScript("OnEvent", function(self, event, ...)
	if not IsInGuild() then
		return;
	end
	GuildWho_Debug("Event fired: "..event);
	if event == "CHAT_MSG_SYSTEM" then
		local inGuild = false;
		local name, online, memberChange = GuildWho_ParseSystemMessage(select(1, ...));
		
		if not name or memberChange then
			return;
		end
		
		local showOffline = GetGuildRosterShowOffline();
		
		SetGuildRosterShowOffline(true);
		GuildWho_Debug("Showing offline members.");
		
		for i = 1, GetNumGuildMembers(true) do
			if GetGuildRosterInfo(i) == name then
				inGuild = true;
				break;
			end
		end
		
		SetGuildRosterShowOffline(showOffline);
		GuildWho_Debug("Restoring display of offline members.");
		
		GuildWho_Debug("inGuild ("..name..") = "..tostring(inGuild));
		if inGuild then
			isReallyOnline[name] = online;
			GuildWho_Debug("isReallyOnline["..name.."] = "..tostring(isReallyOnline[name]));
		end
	-- elseif event == "PLAYER_GUILD_UPDATE" then
		-- if not (GetNumGuildMembers(true) == origGuildies or GetNumGuildMembers(true) == 0) then
			-- frame:RegisterEvent("GUILD_ROSTER_UPDATE");
		-- end
		-- GuildRoster();
	elseif event == "GUILD_ROSTER_UPDATE" then
		local numGuildies = GetNumGuildMembers(true);
		GuildWho_Debug(GetNumGuildMembers(true).. " guild members.");
		if numGuildies > 0 then
			frame:UnregisterEvent("GUILD_ROSTER_UPDATE");
			GuildWho_UpdateGuildRoster();
		end
	end
end);
frame:RegisterEvent("CHAT_MSG_SYSTEM");
frame:RegisterEvent("GUILD_ROSTER_UPDATE");
--frame:RegisterEvent("PLAYER_GUILD_UPDATE");